package pl.otros.logview.filter;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;

import org.apache.commons.io.IOUtils;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;
import pl.otros.logview.gui.PopupListener;

public class ClassFilter implements LogFilter {

    private JTree tree;
    private DefaultMutableTreeNode rootNode;
    private HashMap<Clazz, DefaultMutableTreeNode> clazzNodeMap;
    private boolean enable = false;
    private LogFilterValueChangeListener listener;
    private static final String NAME = "Class Filter";
    private LogDataCollector collector;
    private HashSet<String> ignoreList = new HashSet<String>();
//    private Collection<String> focusList = new ArrayList<String>();
    private JMenuItem addIgnoreMenuItem;
    private JMenuItem removeIgnoreMenuItem;
    private Mode mode = Mode.INGORE_MODE;

    private enum Mode {
        INGORE_MODE, FOCUS_MODE
    }

    @Override
    public boolean accept(LogData logData, int row, boolean marked) {
        String clazz = logData.getClazz();
        if (mode == Mode.INGORE_MODE) {
            for (String c : ignoreList) {
                if (clazz.startsWith(c)) {
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public Component getGUI() {
        return tree;
    }

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    public void init(Properties properties, LogDataCollector collector) {
        this.collector = collector;

        initTree();

    }

    @Override
    public boolean isEnable() {
        return enable;
    }

    @Override
    public void setEnable(boolean enable) {
        this.enable = enable;
        if (enable == true) {
            reloadClasses();
        }
    }

    @Override
    public void setValueChangeListener(LogFilterValueChangeListener listener) {
        this.listener = listener;
    }

    private void initTree() {
        rootNode = new DefaultMutableTreeNode(new Clazz("root"));
        tree = new JTree(rootNode);
        try {
            tree.setCellRenderer(new TreeRenderer());
        } catch (IOException e) {
            System.err.println("Can't lad treerenderer: " + e.getMessage());            
            e.printStackTrace();
        }
        clazzNodeMap = new HashMap<Clazz, DefaultMutableTreeNode>();
        clazzNodeMap.put(new Clazz("root"), rootNode);
        initPopup();
    }

    private void initPopup() {
        JPopupMenu menu = new JPopupMenu("Focusing/Ignoring");
        menu.add(new JLabel("Add package/class to ignore"));
        addIgnoreMenuItem = new JMenuItem("Add to ignore");
        addIgnoreMenuItem.addActionListener(new AddToIgnoreListAction());
        removeIgnoreMenuItem = new JMenuItem("Remove from ignore");
        removeIgnoreMenuItem.addActionListener(new RemoveFromIgnoreListAction());
        menu.add(addIgnoreMenuItem);
        menu.add(removeIgnoreMenuItem);
        tree.addMouseListener(new PopupListener(menu));

    }

    private void reloadClasses() {
        LogData[] ld = collector.getLogData();

        Set<String> classesSet = new TreeSet<String>();
        for (int i = 0; i < ld.length; i++) {
            classesSet.add(ld[i].getClazz());
        }
        
        //prepare packages nodes
        TreeSet<String> packages = new TreeSet<String>();
        for (String string : classesSet) {
            Clazz clazz = new Clazz(string);
            String pack = clazz.pack;
            int start = 0;
            while (start<pack.length()){
                int idx = pack.indexOf('.', start+1);
                if (idx<0){
                    idx=pack.length();
                }
                String upClazz = pack.substring(0, idx);
                packages.add(upClazz);
                start = idx;                
            }
        }
        
        //Create packages tree
        for (String string : packages) {
            Clazz clazz = new Clazz(string);
            DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(clazz);
            DefaultMutableTreeNode parentNode = null;            
            if (clazzNodeMap.containsKey(new Clazz(clazz.pack))) {
                parentNode = clazzNodeMap.get(new Clazz(clazz.pack));
            } else {
                parentNode = rootNode;
            }
            clazzNodeMap.put(clazz, newNode);
            parentNode.add(newNode);
        }

        //Create classes leafs
        for (String string : classesSet) {
            Clazz clazz = new Clazz(string);
            DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(clazz);
            DefaultMutableTreeNode parentNode = null;
            Clazz paketClazz = new Clazz(clazz.pack);
            if (clazzNodeMap.containsKey(paketClazz)) {
                parentNode = clazzNodeMap.get(paketClazz);
            } else {
                parentNode = new DefaultMutableTreeNode(paketClazz);
                rootNode.add(parentNode);
                clazzNodeMap.put(paketClazz, parentNode);
            }
            parentNode.add(newNode);
        }

        tree.expandPath(new TreePath(rootNode));
        tree.repaint();
    }

    private static class Clazz implements Comparable<Clazz> {
        private String pack;
        private String clazz;

        public Clazz(String packageClazz) {
            if (packageClazz.length() == 0) {
                pack = "";
                clazz = "";
                return;
            }

            int idx = packageClazz.lastIndexOf('.');

            if (idx > 0) {
                pack = packageClazz.substring(0, idx);
                clazz = packageClazz.substring(idx + 1);
            } else {
                pack = "";
                clazz = packageClazz;
            }
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof Clazz) {
                Clazz clazz2 = (Clazz) obj;
                return pack.equals(clazz2.pack) && this.clazz.equals(clazz2.clazz);
            } else if (obj instanceof String) {

            }
            return false;
        }

        @Override
        public int hashCode() {
            return clazz.hashCode();
        }

        @Override
        public String toString() {
            return clazz;
        }

        @Override
        public int compareTo(Clazz o) {
            if (pack.equalsIgnoreCase(o.pack)) {
                return clazz.compareTo(o.clazz);
            } else {
                return pack.compareTo(o.pack);
            }
        }

        public String toFullString() {
            if (pack != null && pack.length() > 0) {
                return pack + '.' + clazz;
            } else {
                return clazz;
            }
        }
    }

    private class AddToIgnoreListAction implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            TreePath[] selectedPaths = tree.getSelectionModel().getSelectionPaths();
            for (TreePath treePath : selectedPaths) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) treePath.getLastPathComponent();
                Clazz userObject = (Clazz) node.getUserObject();
                ignoreList.add(userObject.toFullString());                
            }
            listener.valueChanged();
        }

    }

    public class RemoveFromIgnoreListAction implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            TreePath[] selectedPaths = tree.getSelectionModel().getSelectionPaths();
            for (TreePath treePath : selectedPaths) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) treePath.getLastPathComponent();
                Clazz userObject = (Clazz) node.getUserObject();
                ignoreList.remove(userObject.toFullString());                
            }
            listener.valueChanged();
        }

    }

    private class TreeRenderer extends DefaultTreeCellRenderer {

        private ImageIcon clazzIcon;
        private ImageIcon clazzIngoredIcon;
        private ImageIcon packageOpenIcon;
        private ImageIcon packageOpenIgnoreIcon;
        private ImageIcon packageClosedIcon;
        private ImageIcon packageClosedgnoreIcon;

        public TreeRenderer() throws IOException {
            clazzIcon = new ImageIcon(IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("img/class.png")));
            clazzIngoredIcon = new ImageIcon(IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("img/class_ignore.png")));
            packageOpenIcon = new ImageIcon(IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("img/package_open.png")));
            packageOpenIgnoreIcon = new ImageIcon(IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("img/package_open_ignore.png")));
            packageClosedIcon = new ImageIcon(IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("img/package_closed.png")));
            packageClosedgnoreIcon = new ImageIcon(IOUtils.toByteArray(this.getClass().getClassLoader().getResourceAsStream("img/package_closed_ignore.png")));
        }

        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean focus) {
            Component parent = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, focus);
            Clazz clazz = (Clazz) ((DefaultMutableTreeNode) value).getUserObject();
            Font font = getFont();
            JLabel l = (JLabel) parent;
            Icon icon = l.getIcon();
            int style = Font.PLAIN;
            boolean ignore = ignoreList.contains(clazz.toFullString()); 
            if (ignore) {
                style = Font.ITALIC;
            }
            
            if (leaf && ignore) {
                icon = clazzIngoredIcon;
            } else if (leaf) {
                icon = clazzIcon;
            } else if (expanded && ignore){
                icon = packageOpenIgnoreIcon;
            } else if (expanded) {
                icon = packageOpenIcon;
            } else if (!expanded && ignore){
                icon = packageClosedgnoreIcon;
            } else if (!expanded) {
                icon = packageClosedIcon;
            }
            Font newFont = font.deriveFont(style);
            parent.setFont(newFont);
            l.setIcon(icon);
            return parent;
        }
    }
}
