package pl.otros.logview.filter;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import java.util.logging.Level;

import javax.swing.JComboBox;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;
import pl.otros.logview.gui.renderers.LevelRenderer;


public class LevelFilter implements LogFilter {

    private LogFilterValueChangeListener listener;
    private int passLevel = Level.ALL.intValue();
    private boolean enable = false;
    private JComboBox gui;
    private static final String NAME = "Level filter";
    private static LevelRenderer renderer = new LevelRenderer();

    public LevelFilter() {
        gui = new JComboBox(new Level[] { Level.FINEST, Level.FINER, Level.FINE, Level.CONFIG, Level.INFO, Level.WARNING,
                Level.SEVERE });
        gui.setEditable(true);
        gui.setRenderer(renderer);
        gui.setOpaque(true);
        gui.setEditable(false);
        gui.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                gui.setBackground(renderer.getListCellRendererComponent(null, gui.getSelectedItem(), 0, false, false)
                        .getBackground());
                passLevel = ((Level) gui.getSelectedItem()).intValue();
                listener.valueChanged();
            }

        });
    }

    public int getPassLevel() {
        return passLevel;
    }

    public void setPassLevel(int passLevel) {
        this.passLevel = passLevel;
        if (listener != null) {
            listener.valueChanged();
        }
    }

    @Override
    public boolean accept(LogData logData, int row, boolean marked) {
        if (logData.getLevel() == null)
            return true;
        return logData.getLevel().intValue() >= passLevel;
    }

    @Override
    public boolean isEnable() {
        return enable;

    }

    @Override
    public void setEnable(boolean enable) {
        this.enable = enable;
    }

    @Override
    public Component getGUI() {
        return gui;
    }

    @Override
    public void setValueChangeListener(LogFilterValueChangeListener listener) {
        this.listener = listener;

    }

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    public void init(Properties properties, LogDataCollector collector) {

    }

}
