package pl.otros.logview.filter;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Properties;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;

public class ThreadFilter implements LogFilter {

    private boolean enable = false;
    private static final String NAME = "Thread Filter";
    private LogDataCollector collector;
    private LogFilterValueChangeListener listener;
    private JComboBox gui;
    private HashSet<String> threads;
    private DefaultComboBoxModel boxModel;
    private static final String ALL_THREADS = "--ALL--";

    public ThreadFilter() {
        boxModel = new DefaultComboBoxModel();
        boxModel.addElement(ALL_THREADS);
        gui = new JComboBox(boxModel);
        threads = new HashSet<String>();

        gui.setOpaque(true);
        gui.setEditable(false);
        gui.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                listener.valueChanged();
            }
        });

    }

    @Override
    public boolean accept(LogData logData, int row, boolean marked) {
        if (gui.getSelectedIndex() == 0 || logData.getThread().equals(boxModel.getSelectedItem())) {
            return true;
        }
        return false;
    }

    @Override
    public Component getGUI() {
        return gui;
    }

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    public void init(Properties properties, LogDataCollector collector) {
        this.collector = collector;

    }

    @Override
    public boolean isEnable() {
        return enable;
    }

    @Override
    public void setEnable(boolean enable) {
        this.enable = enable;
        if (enable) {
            relaodThreads();
        }
    }

    @Override
    public void setValueChangeListener(LogFilterValueChangeListener listener) {
        this.listener = listener;

    }

    private void relaodThreads() {
        LogData[] ld = collector.getLogData();
        threads.clear();
        for (LogData logData : ld) {
            threads.add(logData.getThread());
        }

        ArrayList<String> toRemoveList = new ArrayList<String>();
        for (int index = 1; index < boxModel.getSize(); index++) {
            String thread = (String) boxModel.getElementAt(index);
            if (!threads.contains(thread)) {
                toRemoveList.add(thread);
            }
        }
        for (String toRemove : toRemoveList) {
            boxModel.removeElement(toRemove);
        }

        for (String thread : threads) {
            if (boxModel.getIndexOf(thread) == -1) {
                boxModel.addElement(thread);
            }
        }
    }

}
