package pl.otros.logview.filter;

import java.awt.Component;
import java.awt.GridLayout;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerDateModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;


public class TimeFilter implements LogFilter {

    private boolean enable = false;
    private static final String NAME = "Time filter";
    private LogFilterValueChangeListener listener;
    private Date start, end;
    private SpinnerDateModel startM, endM;
    private JPanel gui;
    private ChangeListner changeListner;
    private static final DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public TimeFilter() {
        changeListner = new ChangeListner();
        start = new Date();
        end = new Date();
        startM = new SpinnerDateModel();
        startM.setValue(start);
        startM.addChangeListener(changeListner);
        endM = new SpinnerDateModel();
        endM.setValue(end);
        endM.addChangeListener(changeListner);
        gui = new JPanel(new GridLayout(2, 1));
        JSpinner startSpinner = new JSpinner(startM);
        startSpinner.setBorder(BorderFactory.createTitledBorder("Start date"));
        JSpinner endSpinner = new JSpinner(endM);
        endSpinner.setBorder(BorderFactory.createTitledBorder("End date"));
        gui.add(startSpinner);
        gui.add(endSpinner);
    }

    @Override
    public boolean accept(LogData logData, int row, boolean mark) {
        Date d = logData.getDate();
        return d.after(start) && d.before(end);
    }

    @Override
    public Component getGUI() {

        return gui;
    }

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    public boolean isEnable() {
        return enable;
    }

    @Override
    public void setEnable(boolean enable) {
        this.enable = enable;
    }

    @Override
    public void setValueChangeListener(LogFilterValueChangeListener listener) {
        this.listener = listener;
        changeListner.setChangeListener(listener);
    }

    @Override
    public void init(Properties properties, LogDataCollector collector) {
        // Date start = collector.getFirst().getDate();
        // Date end = collector.getLast().getDate();
        // startM.setValue(start);
        // endM.setValue(end);

    }

    private class ChangeListner implements ChangeListener {
        LogFilterValueChangeListener changeListener;

        public LogFilterValueChangeListener getChangeListener() {
            return changeListener;
        }

        public void setChangeListener(LogFilterValueChangeListener changeListener) {
            this.changeListener = changeListener;
        }

        @Override
        public void stateChanged(ChangeEvent e) {
            start = startM.getDate();
            end = endM.getDate();
            listener.valueChanged();

        }

    }

}
