package pl.otros.logview.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

public class FrameManger {

    private static FrameManger instance;
    private JDesktopPane desktopPane;
    private Map<JInternalFrame, JMenuItem> frameMenuItemMap;
    private Map<JInternalFrame, LogViewInternalFrame> frameLogViewFrameMap;
    private JMenu menu;
    private InternalFrameListener frameListener;
    private StatusObserver statusObserver;

    private FrameManger() {
        menu = new JMenu("Windows");
        menu.add(new JLabel("Windows list"));
        menu.add(new JSeparator());
        frameMenuItemMap = new HashMap<JInternalFrame, JMenuItem>();
        frameLogViewFrameMap = new HashMap<JInternalFrame, LogViewInternalFrame>();
    }

    public static FrameManger getInstance() {
        if (instance == null) {
            synchronized (FrameManger.class) {
                if (instance == null) {
                    instance = new FrameManger();
                }
            }
        }
        return instance;
    }

    public LogViewInternalFrame createFrame(String title) {
        LogViewInternalFrame frame = new LogViewInternalFrame(title, desktopPane, statusObserver);
        frame.getIframe().addInternalFrameListener(new FrameListener());
        if (frameListener != null) {
            frame.getIframe().addInternalFrameListener(frameListener);
        }
        JMenuItem menuItem = new JMenuItem(frame.toString());
        menuItem.addActionListener(new FrameFocusListener(frame.getIframe()));
        menu.add(menuItem);

        frameMenuItemMap.put(frame.getIframe(), menuItem);
        frameLogViewFrameMap.put(frame.getIframe(), frame);
        frame.getIframe().setMaximizable(true);
        frameListener.internalFrameOpened(new InternalFrameEvent(frame.getIframe(), InternalFrameEvent.INTERNAL_FRAME_OPENED));
        return frame;
    }

    public JMenu getMenu() {
        return menu;
    }

    public JDesktopPane getDesktopPane() {
        return desktopPane;
    }

    public void setDesktopPane(JDesktopPane desktopPane) {
        this.desktopPane = desktopPane;
    }

    public LogViewInternalFrame getLogViewInternalFrame(JInternalFrame frame) {
        return frameLogViewFrameMap.get(frame);
    }

    public LogViewInternalFrame getActiveLogViewInternalFrame() {
        JInternalFrame frame = desktopPane.getSelectedFrame();
        if (frame != null) {
            return frameLogViewFrameMap.get(frame);
        }
        return null;
    }

    private class FrameListener extends InternalFrameAdapter {

        @Override
        public void internalFrameClosed(InternalFrameEvent e) {
            String title = e.getInternalFrame().getTitle();
            menu.remove(frameMenuItemMap.get(e.getInternalFrame()));
        }
    }

    private class FrameFocusListener implements ActionListener {

        public FrameFocusListener(JInternalFrame frame) {
            super();
            this.frame = frame;
        }

        private JInternalFrame frame;

        @Override
        public void actionPerformed(ActionEvent e) {
            desktopPane.setSelectedFrame(frame);
            desktopPane.getDesktopManager().activateFrame(frame);
            desktopPane.getDesktopManager().deiconifyFrame(frame);
            desktopPane.getDesktopManager().openFrame(frame);

        }

    }

    public InternalFrameListener getFrameListener() {
        return frameListener;
    }

    public void setFrameListener(InternalFrameListener frameListener) {
        this.frameListener = frameListener;
    }

    public StatusObserver getStatusObserver() {
        return statusObserver;
    }

    public void setStatusObserver(StatusObserver statusObserver) {
        this.statusObserver = statusObserver;
    }

}
