package pl.otros.logview.gui;

import java.util.Date;
import java.util.TreeMap;
import java.util.logging.Level;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;


public class LogDataTableModel extends AbstractTableModel implements LogDataCollector, MarkableTableModel {

    private int dataLimit = 199250;
    private TreeMap<Integer, LogData> list;
    private String[] headers = new String[] { "No.", "Time", "Level", "Message", "Class", "Method","Thread", "Mark" };
    public static final int COLUMN_NUMBER = 0;
    public static final int COLUMN_TIME = 1;
    public static final int COLUMN_LEVEL = 2;
    public static final int COLUMN_MESSAGE = 3;
    public static final int COLUMN_CLASS = 4;
    public static final int COLUMN_METHOD = 5;
    public static final int COLUMN_THREAD = 6;
    public static final int COLUMN_MARK = 7;
    private TreeMap<Integer, Boolean> marks;

    public String getColumnName(int column) {
        return headers[column];
    }

    public LogDataTableModel() {
        list = new TreeMap<Integer, LogData>();
        marks = new TreeMap<Integer, Boolean>();

    }

    public int getColumnCount() {
        return headers.length;
    }

    public int getRowCount() {
        return list.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        LogData ld = list.get(rowIndex);
        if (ld == null){
           System.err.println("LogDataTableModel.getValueAt() null form row " + rowIndex);
        }
        Object result = null;        
        switch (columnIndex) {
        case COLUMN_NUMBER:
            result = ld.getId();
            break;
        case COLUMN_TIME:
            result = ld.getDate();
            break;
        case COLUMN_LEVEL:
            result = ld.getLevel();
            break;
        case COLUMN_MESSAGE:
            result = ld.getMessage();
            break;
        case COLUMN_CLASS:
            result = ld.getClazz();
            break;
        case COLUMN_METHOD:
            result = ld.getMethod();
            break;
        case COLUMN_THREAD:
            result = ld.getThread();
            break;
        case COLUMN_MARK:
            result = marks.get(rowIndex);
            break;
        default:
            break;
        }
        return result;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        if (columnIndex == COLUMN_MARK) {
            return true;
        }
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see pl.otros.logview.gui.LogDataCollector#add(pl.otros.logview.LogData[])
     */
    public void add(LogData[] logDatas) {
        for (int i = 0; i < logDatas.length; i++) {
            list.put(logDatas[i].getId(), logDatas[i]);
            marks.put(logDatas[i].getId(), false);
            if (list.size() > dataLimit) {
                list.remove(list.firstKey());
                marks.remove(list.firstKey());
            }
        }
        fireTableDataChanged();
    }

    /*
     * (non-Javadoc)
     * 
     * @see pl.otros.logview.gui.LogDataCollector#add(pl.otros.logview.LogData)
     */
    public void add(LogData logData) {
        list.put(logData.getId(), logData);
        marks.put(logData.getId(), false);
        if (list.size() > dataLimit) {
            list.remove(list.firstKey());
            marks.remove(list.firstKey());
        }
        fireTableDataChanged();
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        // return super.getColumnClass(columnIndex); LogData ld = list.get(rowIndex);
        Class result = String.class;
        switch (columnIndex) {
        case COLUMN_NUMBER:
            result = Integer.class;
            break;
        case COLUMN_TIME:
            result = Date.class;
            break;
        case COLUMN_LEVEL:
            result = Level.class;
            break;
        case COLUMN_MESSAGE:
            result = String.class;
            break;
        case COLUMN_MARK:
            result = Boolean.class;
            break;
        default:
            result = String.class;
            break;
        }
        return result;
    }

    public LogData getLogData(int row) {
        return list.get(row);
    }

    public LogData[] getLogData() {
        LogData[] datas = new LogData[list.size()];
        datas = list.values().toArray(datas);
        return datas;
    }

    @Override
    public LogData getFirst() {
        return list.get(list.firstKey());
    }

    @Override
    public LogData getLast() {
        return list.get(list.lastKey());
    }

    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        if (columnIndex == COLUMN_MARK) {
            marks.put(rowIndex, (Boolean) value);
        }
        fireTableChanged(new TableModelEvent(this, rowIndex));
    }

    @Override
    public boolean isMarked(int row) {
        return marks.get(row);
    }

    @Override
    public void markRows(int[] rows) {
        for (int i = 0; i < rows.length; i++) {
            marks.put(rows[i], true);
        }
        fireTableDataChanged();
    }

    @Override
    public void markRows(int row) {
        marks.put(row, true);
        fireTableDataChanged();
    }

    @Override
    public void unmarkRows(int[] rows) {
        for (int i = 0; i < rows.length; i++) {
            marks.put(rows[i], false);
        }
        fireTableDataChanged();
    }

    @Override
    public void unmarkRows(int row) {
        marks.put(row, false);
        fireTableDataChanged();
    }

}
