package pl.otros.logview.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import javax.imageio.ImageIO;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import pl.otros.logview.gui.actions.AboutAction;
import pl.otros.logview.gui.actions.CloseAllWindowsActionListener;
import pl.otros.logview.gui.actions.CreateUMLActionListener;
import pl.otros.logview.gui.actions.ImportLogActionListener;
import pl.otros.logview.gui.actions.JumpToMarkedAction;
import pl.otros.logview.gui.actions.MaximizeAllWindowsActionListener;
import pl.otros.logview.gui.actions.SearchAction;
import pl.otros.logview.gui.actions.ShowStringRegexMarkerEditorActionListener;
import pl.otros.logview.gui.actions.StartSocketListener;
import pl.otros.logview.gui.actions.JumpToMarkedAction.Direction;
import pl.otros.logview.importer.InitializationException;
import pl.otros.logview.importer.LogImporter;
import pl.otros.logview.loader.LvDynamicLoader;

public class LogViewMainFrame extends JFrame {

    private JToolBar toolBar;
    private JDesktopPane desktopPane;

    private ImportLogActionListener[] importers;
    private JButton buttonSearch;
    private JLabelStatusObserver observer;

    private FrameManger frameManger;

    /**
     * @param args
     * @throws InitializationException
     * @throws InvocationTargetException
     * @throws InterruptedException
     */
    public static void main(String[] args) throws InitializationException, InterruptedException, InvocationTargetException {
        SwingUtilities.invokeAndWait(new Runnable() {

            @Override
            public void run() {
                //                  
                try {
                    final LogViewMainFrame mf = new LogViewMainFrame();
                } catch (InitializationException e) {
                    e.printStackTrace();
                }

            }
        });

    }

    public LogViewMainFrame() throws InitializationException {
        super("OtrosLogViewer");
        this.setSize(800, 700);
        this.setLocation(100, 100);
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);

        try {
            BufferedImage icon = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("img/icon.png"));
            this.setIconImage(icon);
        } catch (Exception e1) {
            System.err.println("Can't load icon: " + e1.getMessage());
        }
        try {
            LvDynamicLoader.getInstance().loadAll();
        } catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, "Problem with loading automatic markers, filter or log imprters: "
                    + e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
        }

        desktopPane = new JDesktopPane();
        desktopPane.setOpaque(true);
        desktopPane.setBorder(BorderFactory.createTitledBorder("Frames"));

        frameManger = FrameManger.getInstance();
        frameManger.setDesktopPane(desktopPane);
        frameManger.setFrameListener(new FrameListener());

        JLabel statusLabel = new JLabel(" ");
        observer = new JLabelStatusObserver(statusLabel);
        frameManger.setStatusObserver(observer);

        initLiseners();
        initMenu();
        initToolbar();

        Container cp = getContentPane();
        cp.setLayout(new BorderLayout());
        cp.add(toolBar, BorderLayout.NORTH);
        cp.add(desktopPane, BorderLayout.CENTER);
        cp.add(statusLabel, BorderLayout.SOUTH);

        setVisible(true);
    }

    private void initToolbar() {
        toolBar = new JToolBar();
        JButton[] importButtons = new JButton[importers.length];
        for (int i = 0; i < importers.length; i++) {
            JButton openLog = new JButton("Open " + importers[i].getImporter().getDisplayableName() + " log");
            openLog.addActionListener(importers[i]);
            if (importers[i].getImporter().getMnemonic() > 0) {
                openLog.setMnemonic(importers[i].getImporter().getMnemonic());
            }
            Icon icon = importers[i].getImporter().getIcon();
            if (icon != null) {
                openLog.setIcon(icon);
            }
            importButtons[i] = openLog;
        }

        final SearchAction searchAction = new SearchAction(observer);
        JTextField searchField = new JTextField(10);
        searchField.setMinimumSize(new Dimension(100, 10));
        searchField.addKeyListener(searchAction);
        final JCheckBox markFouned = new JCheckBox("Mark founded");
        markFouned.setSelected(searchAction.isMarkFounded());
        markFouned.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                searchAction.setMarkFounded(markFouned.isSelected());
            }

        });

        buttonSearch = new JButton("Search Next");
        buttonSearch.setEnabled(false);
        searchAction.setSearchStringFiedl(searchField);
        buttonSearch.addActionListener(searchAction);
        buttonSearch.setMnemonic(KeyEvent.VK_N);
        toolBar.add(new JLabel("Search string: "));
        toolBar.add(searchField);
        toolBar.add(buttonSearch);
        toolBar.add(markFouned);
        
        toolBar.add(new JButton(new JumpToMarkedAction(Direction.FORWARD, observer)));
        toolBar.add(new JButton(new JumpToMarkedAction(Direction.BACKWARD, observer)));
        
        toolBar.add(new JSeparator(JSeparator.VERTICAL));

        for (int i = 0; i < importButtons.length; i++) {
            toolBar.add(importButtons[i]);
        }

    }

    private void initLiseners() throws InitializationException {
        Collection<LogImporter> impList = LvDynamicLoader.getInstance().getLogImporters();
        importers = new ImportLogActionListener[impList.size()];
        int i = 0;
        for (LogImporter logImporter : impList) {
            importers[i] = new ImportLogActionListener(desktopPane, logImporter);
            i++;
        }
    }

    private void initMenu() {
        JMenuBar menuBar = new JMenuBar();

        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);

        for (int i = 0; i < importers.length; i++) {
            JMenuItem openLog = new JMenuItem("Open " + importers[i].getImporter().getDisplayableName() + " log");
            openLog.addActionListener(importers[i]);
            if (importers[i].getImporter().getKeyStrokeAccelelator() != null) {
                openLog.setAccelerator(KeyStroke.getKeyStroke(importers[i].getImporter().getKeyStrokeAccelelator()));
            }
            if (importers[i].getImporter().getMnemonic() > 0) {
                openLog.setMnemonic(importers[i].getImporter().getMnemonic());
            }
            Icon icon = importers[i].getImporter().getIcon();
            if (icon != null) {
                openLog.setIcon(icon);
            }
            fileMenu.add(openLog);
        }

        JMenuItem exitMenuItem = new JMenuItem("Exit", 'e');
        exitMenuItem.setAccelerator(KeyStroke.getKeyStroke("control F4"));
        exitMenuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });

        fileMenu.add(new JSeparator());
        fileMenu.add(exitMenuItem);

        JMenu windowsMenu = frameManger.getMenu();
        windowsMenu.setMnemonic(KeyEvent.VK_W);
        JMenuItem closeAll = new JMenuItem("Close all");
        CloseAllWindowsActionListener closeAllAction = new CloseAllWindowsActionListener(frameManger);
        closeAll.addActionListener(closeAllAction);
        windowsMenu.add(closeAll, 0);

        MaximizeAllWindowsActionListener maxAllWindows = new MaximizeAllWindowsActionListener(frameManger);
        JMenuItem maxAll = new JMenuItem("Maximize all windows");
        maxAll.addActionListener(maxAllWindows);
        windowsMenu.add(maxAll, 0);

        JMenu toolsMenu = new JMenu("Tools");
        toolsMenu.setMnemonic(KeyEvent.VK_T);
        JMenuItem umlMenuItem = new JMenuItem("Create UML", 'U');
        umlMenuItem.addActionListener(new CreateUMLActionListener());
        JMenuItem markerEditor = new JMenuItem("String and regex marker editor");
        markerEditor.addActionListener(new ShowStringRegexMarkerEditorActionListener(this));
        
        
        toolsMenu.add(umlMenuItem);
        toolsMenu.add(new JMenuItem(new StartSocketListener(observer)));
        toolsMenu.add(markerEditor);

        
        
        JMenu helpMenu = new JMenu("Help");
        JMenuItem about = new JMenuItem("About");
        AboutAction action = new AboutAction(this);
        action.putValue(Action.NAME, "About");
        about.setAction(action);
        helpMenu.add(about);

        menuBar.add(fileMenu);
        menuBar.add(toolsMenu);
        menuBar.add(windowsMenu);
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);

    }

    private class FrameListener extends InternalFrameAdapter {

        @Override
        public void internalFrameActivated(InternalFrameEvent e) {
            buttonSearch.setEnabled(true);
        }

        @Override
        public void internalFrameClosed(InternalFrameEvent e) {
            JInternalFrame frame = desktopPane.getSelectedFrame();
            if (frame == null) {
                buttonSearch.setEnabled(false);
            }
        }

        @Override
        public void internalFrameOpened(InternalFrameEvent e) {
            this.internalFrameActivated(e);
        }
    }

}
