package pl.otros.logview.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableRowSorter;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;
import pl.otros.logview.filter.FilterPanel;
import pl.otros.logview.filter.LogFilter;
import pl.otros.logview.filter.LogFilterValueChangeListener;
import pl.otros.logview.gui.actions.AutomaticMarkUnamrkActionListener;
import pl.otros.logview.gui.actions.MarkRowAction;
import pl.otros.logview.gui.actions.ShowCallHierarhyAction;
import pl.otros.logview.gui.actions.TableResizeActionListener;
import pl.otros.logview.gui.actions.UnMarkRowAction;
import pl.otros.logview.gui.markers.AutomaticMarker;
import pl.otros.logview.gui.renderers.DateRenderer;
import pl.otros.logview.gui.renderers.LevelRenderer;
import pl.otros.logview.gui.renderers.TableMarkDecoratorRenderer;
import pl.otros.logview.loader.LvDynamicLoader;


public class LogViewPanel extends JPanel implements LogDataCollector {

    private JPanel filtersPanel;
    private JPanel logsTablePanel;

    private LogDataTableModel dataTableModel;
    private JTextArea logDetailTextArea;
    private JTable table;
    private TableRowSorter<LogDataTableModel> sorter;
    private StatusObserver statusObserver;
    private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss.SSS");

    public LogViewPanel(final LogDataTableModel dataTableModel, StatusObserver statusObserver) {
        super();
        this.dataTableModel = dataTableModel;
        this.statusObserver = statusObserver;
        filtersPanel = new JPanel();
        logsTablePanel = new JPanel();
        // pagingTableModel = new PagingTableDataModel(dataTableModel);
        logDetailTextArea = new JTextArea();
        logDetailTextArea.setLineWrap(true);
        logDetailTextArea.setEditable(false);
        logDetailTextArea.setBorder(BorderFactory.createTitledBorder("Details"));
        // table = new JTable(pagingTableModel);
        table = new JTable(dataTableModel);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.getColumnModel().getColumn(0).setMaxWidth(100);
        table.getColumnModel().getColumn(0).setWidth(40);        
        table.getColumnModel().getColumn(1).setWidth(100);
        table.getColumnModel().getColumn(2).setMaxWidth(70);
        table.getColumnModel().getColumn(2).setMinWidth(70);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        table.setDefaultRenderer(Object.class, new TableMarkDecoratorRenderer(table.getDefaultRenderer(Object.class)));
        table.setDefaultRenderer(Integer.class, new TableMarkDecoratorRenderer(table.getDefaultRenderer(Object.class)));
        table.setDefaultRenderer(Level.class, new LevelRenderer());
        table.setDefaultRenderer(Date.class, new TableMarkDecoratorRenderer(new DateRenderer()));
        sorter = new TableRowSorter<LogDataTableModel>(dataTableModel);
        for (int i = 0; i < dataTableModel.getColumnCount(); i++) {
            sorter.setSortable(i,false);
        }
        sorter.setSortable(LogDataTableModel.COLUMN_NUMBER, true);
        sorter.setSortable(LogDataTableModel.COLUMN_TIME, true);
        table.setRowSorter(sorter);
        table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent e) {
                int row = table.getSelectedRow();
                if (row >= 0 && row < table.getRowCount()) {
                    logDetailTextArea.setText("");
                    LogData ld = dataTableModel.getLogData(table.convertRowIndexToModel(row));
                    logDetailTextArea.append("Date:    " + dateFormat.format(ld.getDate()) + "\n");
                    logDetailTextArea.append("Class:   " + ld.getClazz() + "\n");
                    logDetailTextArea.append("Method:  " + ld.getMethod() + "\n");
                    logDetailTextArea.append("Message: " + ld.getMessage());
                }
            }

        });

        addFiltersGUIsToPanel(filtersPanel);
        logsTablePanel.setLayout(new BorderLayout());
        logsTablePanel.add(new JScrollPane(table));
        JSplitPane splitPaneLogsTableAndDetails = new JSplitPane(JSplitPane.VERTICAL_SPLIT, logsTablePanel, new JScrollPane(logDetailTextArea));
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, new JScrollPane(filtersPanel,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), splitPaneLogsTableAndDetails);
        splitPane.setOneTouchExpandable(true);
        this.setLayout(new BorderLayout());
        this.add(splitPane);
        
        splitPaneLogsTableAndDetails.setDividerLocation(0.5d);
        splitPaneLogsTableAndDetails.setOneTouchExpandable(true);
        splitPane.setDividerLocation(filtersPanel.getPreferredSize().width+10);

        initTableContextMenu();
    }

    public void add(LogData[] logDatas) {
        dataTableModel.add(logDatas);

    }

    public void add(LogData logData) {
        dataTableModel.add(logData);
    }

    public LogData[] getLogData() {
        return dataTableModel.getLogData();
    }

    private void addFiltersGUIsToPanel(JPanel panel) {
        GridBagLayout bagLayout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.anchor = GridBagConstraints.NORTH;
        constraints.gridx = 0;
        constraints.gridy = GridBagConstraints.REMAINDER;
        constraints.fill = GridBagConstraints.BOTH;
        panel.setLayout(new BorderLayout());
        JPanel filtersPanel = new  JPanel(bagLayout);
        Collection<LogFilter> loadeFilters = LvDynamicLoader.getInstance().getLogFilters();

        //Reload filters, every instance of filter is connected to listeners, data table etc. 
        Collection<LogFilter> filtersList = new ArrayList<LogFilter>();
        for (LogFilter logFilter : loadeFilters) {
            try {
                filtersList.add(logFilter.getClass().newInstance());
            } catch (Exception e) {
                System.err.println("can't initialize filter: " + logFilter.getClass());
                continue;
            }
            
        }
        JLabel filtersLabel = new JLabel("Filters:");
        Font f = filtersLabel.getFont().deriveFont(Font.BOLD + Font.ITALIC);
        filtersLabel.setFont(f);
        filtersPanel.add(filtersLabel,constraints);
        constraints.gridy++;
        LogFilterValueChangeListener listener = new LogFilterValueChangeListener(sorter, filtersList, statusObserver);
        for (LogFilter filter : filtersList) {
            filter.init(new Properties(), this);
            FilterPanel filterPanel = new FilterPanel(filter, listener);
            constraints.gridy++;
            filtersPanel.add(filterPanel, constraints);
        }
        constraints.fill = GridBagConstraints.VERTICAL;
        constraints.gridy++;
        
        panel.add(filtersPanel,BorderLayout.NORTH);
    }

    public LogData getFirst() {
        return dataTableModel.getFirst();
    }

    public LogData getLast() {
        return dataTableModel.getLast();
    }

    private void initTableContextMenu() {
        JPopupMenu menu = new JPopupMenu("Menu");
        JMenuItem mark = new JMenuItem("Mark selected rows");
        mark.addActionListener(new MarkRowAction(table, dataTableModel, statusObserver));
        JMenuItem unmark = new JMenuItem("Unmark selected rows");
        unmark.addActionListener(new UnMarkRowAction(table, dataTableModel, statusObserver));

        JMenuItem callHierarchy = new JMenuItem("Call hierarchy");
        callHierarchy.addActionListener(new ShowCallHierarhyAction(table, dataTableModel, statusObserver));
        
        JMenuItem autoResizeMenu = new JMenu("Table auto resize mode");
        JMenuItem autoResizeSubseqent = new JMenuItem("Subsequent columns");
        autoResizeSubseqent.addActionListener(new TableResizeActionListener(table, JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS));
        JMenuItem autoResizeLast = new JMenuItem("Last column");
        autoResizeLast.addActionListener(new TableResizeActionListener(table, JTable.AUTO_RESIZE_LAST_COLUMN));
        JMenuItem autoResizeNext = new JMenuItem("Next column");
        autoResizeNext.addActionListener(new TableResizeActionListener(table, JTable.AUTO_RESIZE_NEXT_COLUMN));
        JMenuItem autoResizeAll = new JMenuItem("All columns");
        autoResizeAll.addActionListener(new TableResizeActionListener(table, JTable.AUTO_RESIZE_ALL_COLUMNS));
        JMenuItem autoResizeOff = new JMenuItem("Auto resize off");
        autoResizeOff.addActionListener(new TableResizeActionListener(table, JTable.AUTO_RESIZE_OFF));
        autoResizeMenu.add(autoResizeSubseqent);
        autoResizeMenu.add(autoResizeOff);
        autoResizeMenu.add(autoResizeNext);
        autoResizeMenu.add(autoResizeLast);
        autoResizeMenu.add(autoResizeAll);

        menu.add(new JLabel("Maring/unamrking"));
        menu.add(mark);
        menu.add(unmark);
        JMenu[] markersMenu = getAutomaticMarkersMenu();
        menu.add(markersMenu[0]);
        menu.add(markersMenu[1]);
        menu.add(new JSeparator());
        menu.add(autoResizeMenu);
        menu.add(callHierarchy);
        table.addMouseListener(new PopupListener(menu));
    }

    private JMenu[] getAutomaticMarkersMenu() {
        AutomaticMarker[] markers = LvDynamicLoader.getInstance().getAutomaticMarkers().toArray(new AutomaticMarker[0]);
        JMenu markMenu = new JMenu("Marks ->");
        JMenu unMarkMenu = new JMenu("Unarks ->");
        HashMap<String, JMenu> marksGroups = new HashMap<String, JMenu>();
        HashMap<String, JMenu> unmarksGroups = new HashMap<String, JMenu>();

        for (AutomaticMarker automaticMarker : markers) {
            addMarkerToMenu(markMenu, automaticMarker, marksGroups, AutomaticMarkUnamrkActionListener.MODE_MARK);
            addMarkerToMenu(unMarkMenu, automaticMarker, unmarksGroups, AutomaticMarkUnamrkActionListener.MODE_UNMARK);
        }
        markMenu = GuiUtils.sortDirAndAlfabetic(markMenu);
        unMarkMenu = GuiUtils.sortDirAndAlfabetic(unMarkMenu);
        return new JMenu[] { markMenu, unMarkMenu };

    }

    private void addMarkerToMenu(JMenu menu, AutomaticMarker automaticMarker, HashMap<String, JMenu> marksGroups, boolean mode) {
        String[] groups = automaticMarker.getMarkerGroups();
        if (groups == null || groups.length == 0) {
            groups = new String[] { "" };
        }
        for (String g : groups) {
            JMenuItem markerMenuItem = new JMenuItem(automaticMarker.getName());
            markerMenuItem.setToolTipText(automaticMarker.getDescripion());
            markerMenuItem.addActionListener(new AutomaticMarkUnamrkActionListener(dataTableModel, automaticMarker, mode,
                    statusObserver));
            if (g.length() > 0) {
                JMenu m = marksGroups.get(g);
                if (m == null) {
                    m = new JMenu(g);
                    marksGroups.put(g, m);
                    menu.add(m);
                }
                m.add(markerMenuItem);
            } else {
                menu.add(markerMenuItem);
            }
        }

    }

    public JTable getTable() {
        return table;
    }

}
