package pl.otros.logview.gui.actions;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.NumberFormat;

import javax.swing.JDesktopPane;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

import net.sf.vfsjfilechooser.VFSJFileChooser;
import net.sf.vfsjfilechooser.VFSJFileChooser.RETURN_TYPE;
import net.sf.vfsjfilechooser.VFSJFileChooser.SELECTION_MODE;
import net.sf.vfsjfilechooser.accessories.DefaultAccessoriesPanel;

import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;

import pl.otros.logview.gui.FrameManger;
import pl.otros.logview.gui.LogDataTableModel;
import pl.otros.logview.gui.LogViewInternalFrame;
import pl.otros.logview.importer.LogImporter;
import pl.otros.logview.importer.ObserableInputStreamImpl;

public class ImportLogActionListener implements ActionListener {

    private JDesktopPane desktopPane;
    private VFSJFileChooser chooser;
    private LogImporter importer;

    public ImportLogActionListener(JDesktopPane desktopPane, LogImporter importer) {
        super();
        this.desktopPane = desktopPane;
        this.importer = importer;
    }

    public void actionPerformed(ActionEvent e) {
        if (chooser == null) {
            initFileChooser();
        }
        RETURN_TYPE result = chooser.showOpenDialog((Component) e.getSource());
        if (result != RETURN_TYPE.APPROVE) {
            return;
        }
        final FileObject[] files = chooser.getSelectedFiles();
        for (int i = 0; i < files.length; i++) {
            final int j = i;
            
            final LogViewInternalFrame frame = FrameManger.getInstance().createFrame(files[j].getName().getPath());            
            
            Runnable r = new Runnable() {
                @Override
                public void run() {
                    ProgressWatcher watcher = null;
                    try {
                        LogDataTableModel dataTableModel = frame.getDataTableModel();
                        ObserableInputStreamImpl in;
                        in = new ObserableInputStreamImpl(files[j].getContent().getInputStream());
                        watcher = new ProgressWatcher(in, frame,files[j]);
                        Thread t = new Thread(watcher, "Log loader: " + files[j].getName().toString());
                        t.setDaemon(true);
                        t.start();                        
                        importer.importLogs(in, dataTableModel, true);
                        SwingUtilities.invokeLater(new Runnable() {                           
                            @Override
                            public void run() {
                                frame.switchToContentView();                                
                            }
                        });
                        watcher.updateFinish("Loaded");
                    } catch (FileSystemException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                        JOptionPane.showMessageDialog(null, "Error loadin log: " + e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                    } 

                }
            };
            Thread t = new Thread(r);
            t.start();
        }
    }

    private void initFileChooser() {
        chooser = new VFSJFileChooser();
        chooser.setDialogTitle("Open " + importer.getDisplayableName() + " log");
        chooser.setAccessory(new DefaultAccessoriesPanel(chooser));
        chooser.setFileHidingEnabled(false);
        chooser.setFileSelectionMode(SELECTION_MODE.FILES_ONLY);
        chooser.setMultiSelectionEnabled(true);
    }

    public LogImporter getImporter() {
        return importer;
    }

    public void setImporter(LogImporter importer) {
        this.importer = importer;
    }

    private class ProgressWatcher implements Runnable {

        private ObserableInputStreamImpl in;
        private JProgressBar progressBar;
        private boolean refreshProgress = true;
        private LogViewInternalFrame frame;
        private FileObject fileName;
        private NumberFormat npf = NumberFormat.getPercentInstance();
        private NumberFormat nbf = NumberFormat.getIntegerInstance();

        public ProgressWatcher(ObserableInputStreamImpl in, LogViewInternalFrame frame, FileObject fileName) {
            super();
            this.in = in;
            this.frame = frame;
            this.fileName = fileName;
            this.progressBar = frame.getLoadingProgressBar();
            npf.setMaximumFractionDigits(0);
            nbf.setGroupingUsed(true);
        }

        @Override
        public void run() {
            while (refreshProgress) {
                try {
                    Thread.sleep(100);
                    long max = fileName.getContent().getSize();
                    if (max <= 0) {
                        updateNotDetermined("Loading");
                    } else {
                        long current = in.getCurrentRead();
                        float percent = (float) current/ max;
                        long currentInKb = current/1024;
                        long maxInKb= max/1024;
                        String message = "Loading " + fileName.getName().getBaseName() + " ... " + npf.format(percent) + "["+
                        nbf.format(currentInKb) +"kb of " + nbf.format(maxInKb) +"kb]"; 
                        updateProgress(message, (int) current, 0, (int) max);

                    }

                } catch (InterruptedException e) {
                    // waiting interrupted, nothing serious.
                    
                } catch (IOException e) {
                    // e.printStackTrace();
                    // stream closed - all loaded
                    break;
                }
            }
            updateFinish("Loaded");

        }

        public void updateNotDetermined(final String message) {
            Runnable r = new Runnable() {
                @Override
                public void run() {
                    progressBar.setIndeterminate(true);
                    progressBar.setString(message);
                }

            };

            SwingUtilities.invokeLater(r);
        }

        public void updateProgress(final String message, final int current, final int min, final int max) {
            Runnable r = new Runnable() {

                @Override
                public void run() {
                    progressBar.setIndeterminate(false);
                    progressBar.setMaximum(max);
                    progressBar.setMinimum(min);
                    progressBar.setValue(current);
                    progressBar.setString(message);
//                    refreshProgress = false;

                }

            };
            try {
                SwingUtilities.invokeAndWait(r);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }

        }

        public void updateFinish(final String message) {
            Runnable r = new Runnable() {

                @Override
                public void run() {
                    progressBar.setIndeterminate(false);

                    progressBar.setMaximum(1);
                    progressBar.setMinimum(0);
                    progressBar.setValue(1);
                    progressBar.setString(message);
                    frame.switchToContentView();
                    refreshProgress =false;
                }

            };
            SwingUtilities.invokeLater(r);

        }
    }
}
