package pl.otros.logview.gui.actions;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JTable;

import pl.otros.logview.gui.FrameManger;
import pl.otros.logview.gui.LogDataTableModel;
import pl.otros.logview.gui.LogViewInternalFrame;
import pl.otros.logview.gui.StatusObserver;

public class JumpToMarkedAction extends AbstractAction {

    private StatusObserver observer;
    private Direction direction;

    public enum Direction {
        FORWARD, BACKWARD;
    }

    public JumpToMarkedAction(Direction direction, StatusObserver observer) {
        this.observer = observer;
        this.direction = direction;
        if (direction == Direction.FORWARD) {
            putValue(Action.NAME, "Next marked");
        } else {
            putValue(Action.NAME, "Previous marked");
        }

    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
        LogViewInternalFrame lvFrame = FrameManger.getInstance().getActiveLogViewInternalFrame();
        if (lvFrame == null) {
            observer.updateStatus("Log file not open", StatusObserver.LEVEL_ERROR);
            return;

        }
        JTable table = lvFrame.getLogViewPanel().getTable();
        LogDataTableModel model = lvFrame.getDataTableModel();
        if (table.getRowCount() == 0) {
            observer.updateStatus("Empty log", StatusObserver.LEVEL_ERROR);
            return;
        }
        int startRow = table.getSelectedRow();
        startRow = startRow < 0 ? startRow = 0 : startRow;
        int nextRow = startRow;
        while (true) {
            if (direction == Direction.FORWARD) {
                nextRow++;
            } else {
                nextRow--;
            }
            
            if (nextRow == table.getRowCount()) {
                nextRow = 0;
            } else if (nextRow < 0) {
                nextRow = table.getRowCount()-1;
            }
            
            if (nextRow == startRow) {
                // not found
                observer.updateStatus("Next marked not found", StatusObserver.LEVEL_ERROR);
                return;
            }
            observer.updateStatus("Checking row: " + nextRow);
            int rowInModel = table.convertRowIndexToModel(nextRow);
            System.out.println("JumpToMarkedAction.actionPerformed() table/model: " + nextRow + "/"+rowInModel);
            boolean marked = model.isMarked(table.convertRowIndexToModel(nextRow));
            System.out.println("Marked = " + marked);
            if (marked) {
                Rectangle rect = table.getCellRect(nextRow, 0, true);
                table.scrollRectToVisible(rect);
                table.clearSelection();
                table.setRowSelectionInterval(nextRow, nextRow);
                observer.updateStatus("Next marked found at " + table.convertRowIndexToView(nextRow), StatusObserver.LEVEL_NORMAL);
                System.out.println("JumpToMarkedAction.actionPerformed() exit");
                break;
            }

        }

    }

}
