package pl.otros.logview.gui.actions;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JTable;
import javax.swing.JTextField;

import pl.otros.logview.LogData;
import pl.otros.logview.gui.FrameManger;
import pl.otros.logview.gui.LogDataTableModel;
import pl.otros.logview.gui.LogViewInternalFrame;
import pl.otros.logview.gui.StatusObserver;


public class SearchAction implements ActionListener, KeyListener {

    private LogDataTableModel model;
    private JTable table;
    private JTextField searchStringFiedl;
    private int lastRow = 0;
    private StatusObserver statusObserver;
    private boolean markFounded = false;

    public SearchAction(StatusObserver statusObserver) {
        this.statusObserver = statusObserver;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        LogViewInternalFrame lvFrame = FrameManger.getInstance().getActiveLogViewInternalFrame();
        if (lvFrame == null) {
            return;
        }
        table = lvFrame.getLogViewPanel().getTable();
        model = lvFrame.getDataTableModel();
        if (model.getRowCount() == 0) {
            statusObserver.updateStatus("Empty log", StatusObserver.LEVEL_WARNING);
            return;
        }

        String string = searchStringFiedl.getText().toLowerCase().trim();
        int rowSearchStart = lastRow;

        if (table.getSelectedRow() > 0) {
            lastRow = table.getSelectedRow() + 1;
            rowSearchStart = lastRow;
        } else {
            rowSearchStart = 0;
            lastRow = 0;
        }

        if (lastRow >= model.getRowCount() - 1) {
            lastRow = 0;
            rowSearchStart = 0;
        }

        while (true) {
            LogData ld = model.getLogData(lastRow);
            if (ld.getMessage().toLowerCase().contains(string)) {
                if (markFounded) {
                    model.markRows(lastRow);
                }
                Rectangle rect = table.getCellRect(lastRow, 0, true);
                table.scrollRectToVisible(rect);
                table.clearSelection();
                table.setRowSelectionInterval(lastRow, lastRow);
                lastRow++;
                statusObserver.updateStatus("Founded at row " + (lastRow - 1), StatusObserver.LEVEL_NORMAL);
                break;
            }
            lastRow++;
            if (lastRow >= model.getRowCount()) {
                lastRow = 0;
            }
            if (lastRow == rowSearchStart) {
                lastRow = 0;
                statusObserver.updateStatus("Not found", StatusObserver.LEVEL_ERROR);
                break;
            }
        }
    }

    public JTextField getSearchStringFiedl() {
        return searchStringFiedl;
    }

    public void setSearchStringFiedl(JTextField searchStringFiedl) {
        this.searchStringFiedl = searchStringFiedl;
    }

    public StatusObserver getStatusObserver() {
        return statusObserver;
    }

    public void setStatusObserver(StatusObserver statusObserver) {
        this.statusObserver = statusObserver;
    }

    @Override
    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == 10) {
            actionPerformed(new ActionEvent(e.getSource(), e.getID(), ""));
        }

    }

    @Override
    public void keyReleased(KeyEvent e) {

    }

    @Override
    public void keyTyped(KeyEvent e) {

    }

    public boolean isMarkFounded() {
        return markFounded;
    }

    public void setMarkFounded(boolean markFounded) {
        this.markFounded = markFounded;
    }

}
