package pl.otros.logview.gui.markers;

import java.util.Properties;
import java.util.regex.Pattern;

import pl.otros.logview.LogData;

public class RegexMarker implements AutomaticMarker {

    public static final String PRECONDITION = "precondition";
    public static final String CONDITION = "condition";
    public static final String DESCRIPTION = "description";
    public static final String GROUPS = "groups";
    public static final String NAME = "name";
    public static final String PRECONDITION_INCLUDE = "preconditionInclude";
    public static final String IGNORE_CASE="ignoreCase";

    private String precondition;
    private Pattern condition;
    private String description;
    private String[] markerGroups;
    private String name;
    private boolean preconditionInclude = true;
    private boolean ignoreCase = true;

    public RegexMarker(Properties p) throws Exception {
        if (!p.containsKey(PRECONDITION) || !p.containsKey(CONDITION) || !p.containsKey(DESCRIPTION) || !p.containsKey(GROUPS)
                || !p.containsKey(NAME) || !p.containsKey(PRECONDITION_INCLUDE) || !p.containsKey(IGNORE_CASE)) {
            throw new Exception("Not enought parameters");
        }
        this.ignoreCase = Boolean.parseBoolean(p.getProperty(IGNORE_CASE));
        
        if (ignoreCase){
            this.condition = Pattern.compile(p.getProperty(CONDITION),Pattern.CASE_INSENSITIVE);
            this.precondition = p.getProperty(PRECONDITION).toLowerCase();
        } else {
            this.precondition = p.getProperty(PRECONDITION);
            this.condition = Pattern.compile(p.getProperty(CONDITION));
        }
        
        this.description = p.getProperty(DESCRIPTION);
        this.markerGroups = p.getProperty(GROUPS).split(",");
        for (int i = 0; i < markerGroups.length; i++) {
            markerGroups[i] = markerGroups[i].trim();
        }
        this.name = p.getProperty(NAME);
        this.preconditionInclude = Boolean.parseBoolean(p.getProperty(PRECONDITION_INCLUDE));
    }

    @Override
    public String getDescripion() {
        return description;
    }

    @Override
    public String[] getMarkerGroups() {
        return markerGroups;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public boolean toMark(LogData data) {
        String message = ignoreCase?data.getMessage().toLowerCase():data.getMessage();
        
        // Checking precondition for performance reason
        if (preconditionInclude && !message.contains(precondition) || !preconditionInclude && message.contains(precondition)) {
            return false;
        }
        boolean result = condition.matcher(message).find(); 
        return result;
    }

}
