package pl.otros.logview.gui.markers;

import java.util.Properties;

import pl.otros.logview.LogData;

public class StringMarker implements AutomaticMarker {


    public static final String CONDITION = "condition";
    public static final String DESCRIPTION = "description";
    public static final String GROUPS = "groups";
    public static final String NAME = "name";
    public static final String INCLUDE = "include";
    public static final String IGNORE_CASE="ignoreCase";

    private String condition;
    private String description;
    private String[] markerGroups;
    private String name;
    private boolean include = true;
    private boolean ignoreCase = true;

    public StringMarker(Properties p) throws Exception {
        if (!p.containsKey(CONDITION) || !p.containsKey(DESCRIPTION) || !p.containsKey(GROUPS)
                || !p.containsKey(NAME) || !p.containsKey(IGNORE_CASE)) {
            throw new Exception("Not enought parameters");
        }
        this.ignoreCase = Boolean.parseBoolean(p.getProperty(IGNORE_CASE));
        
        if (ignoreCase){
            this.condition = p.getProperty(CONDITION).toLowerCase();
        } else {
            this.condition = p.getProperty(CONDITION);
        }
        
        this.description = p.getProperty(DESCRIPTION);
        this.markerGroups = p.getProperty(GROUPS).split(",");
        for (int i = 0; i < markerGroups.length; i++) {
            markerGroups[i] = markerGroups[i].trim();
        }
        this.name = p.getProperty(NAME);
        this.include = Boolean.parseBoolean(p.getProperty(INCLUDE));
    }

    @Override
    public String getDescripion() {
        return description;
    }

    @Override
    public String[] getMarkerGroups() {
        return markerGroups;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public boolean toMark(LogData data) {
        String message = ignoreCase?data.getMessage().toLowerCase():data.getMessage();        
        return include && message.contains(condition) || !include && !message.contains(condition);

    }

}
