package pl.otros.logview.gui.renderers;

import java.awt.Color;
import java.awt.Component;
import java.util.logging.Level;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;

public class LevelRenderer implements TableCellRenderer, ListCellRenderer {

    private JLabel label = new JLabel();

    private Color colorSevere = Color.RED;
    private Color colorWarning = Color.ORANGE;
    private Color colorInfo = Color.GREEN;
    private Color colorConfig = new Color(154, 255, 154);
    private Color colorFine = new Color(204, 204, 204);
    private Color colorFiner = new Color(170, 170, 170);
    private Color colorFinest = new Color(136, 136, 136);

    public LevelRenderer() {
        label.setOpaque(true);
    }

    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
            int column) {
        if (value instanceof Level) {
            Level level = (Level) value;
            label.setBackground(getColorByLevel(level));
            label.setText(level.getName());
        } else {
            label.setBackground(Color.WHITE);
            label.setText("");
        }
        return label;
    }

    private Color getColorByLevel(Level level) {
        Color result = Color.WHITE;
        if (level.equals(Level.SEVERE)) {
            result = colorSevere;
        } else if (level.equals(Level.WARNING)) {
            result = colorWarning;
        } else if (level.equals(Level.INFO)) {
            result = colorInfo;
        } else if (level.equals(Level.CONFIG)) {
            result = colorConfig;
        } else if (level.equals(Level.FINE)) {
            result = colorFine;
        } else if (level.equals(Level.FINER)) {
            result = colorFiner;
        } else if (level.equals(Level.FINEST)) {
            result = colorFinest;
        }
        return result;
    }

    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        label.setForeground(Color.BLACK);
        if (value instanceof Level) {
            Level level = (Level) value;
            label.setBackground(getColorByLevel(level));
            label.setText(level.getName());
        } else {
            label.setBackground(Color.WHITE);
            label.setText("");
        }
        if (isSelected) {
            Color c1 = label.getBackground();
            label.setBackground(label.getForeground());
            label.setForeground(c1);
        }
        return label;
    }

}
