package pl.otros.logview.gui.renderers;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import pl.otros.logview.gui.MarkableTableModel;


public class TableMarkDecoratorRenderer implements TableCellRenderer {

    private TableCellRenderer subjectRenderer;

    public TableMarkDecoratorRenderer(TableCellRenderer subjectRenderer) {
        super();
        this.subjectRenderer = subjectRenderer;
    }

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
            int column) {
        Component c = subjectRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        boolean marked = false;
        if (table.getModel() instanceof MarkableTableModel) {
            MarkableTableModel model = (MarkableTableModel) table.getModel();
            try {
                marked = model.isMarked(table.convertRowIndexToModel(row));
            } catch (NullPointerException e) {
                System.err.println("TableMarkDecoratorRenderer.getTableCellRendererComponent() null pointer catched at index " + row);
                e.printStackTrace();
            }
        }

        Color bg = null;
        if (!isSelected && !marked) {
            bg = DefaultColors.BACKGROUND;
        } else if (isSelected && !marked) {
            bg = DefaultColors.SELECTED;
        } else if (marked && !isSelected) {
            bg = DefaultColors.MARKED;
        } else {
            bg = DefaultColors.MARKED_SELECTED;
        }
        c.setBackground(bg);

        return c;
    }

}
