package pl.otros.logview.importer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.Properties;

import javax.swing.Icon;

import pl.otros.logview.LogData;
import pl.otros.logview.LogDataCollector;
import pl.otros.logview.parser.LogParser;
import pl.otros.logview.parser.ParserDescription;
import pl.otros.logview.parser.ParsingContext;
import pl.otros.logview.reader.ProxyLogDataCollector;


public class LogImporterUsingParser implements LogImporter {

    private LogParser parser = null;
    private String displayableName;
    private int mnemonic;
    private String keyStrokeAccelelator;
    private Icon icon;

    public LogImporterUsingParser(LogParser parser) {
        super();
        this.parser = parser;
        ParserDescription pd = parser.getParserDescription();
        displayableName = pd.getDisplayName();
        mnemonic = pd.getMenmonic();
        keyStrokeAccelelator = pd.getKeyStrokeAccelelator();
        icon = pd.getIcon();
    }

    @Override
    public void init(Properties properties) throws InitializationException {

    }

    @Override
    public void importLogs(InputStream in, LogDataCollector dataTableModel, boolean addAllDataOnFinish) {
        ParsingContext parsingContext = new ParsingContext();
        String line = null;
        LogData logData = null;
        LogDataCollector logDataCollector = dataTableModel;
        if (addAllDataOnFinish) {
            logDataCollector = new ProxyLogDataCollector();
        }
        BufferedReader logReader = new BufferedReader(new InputStreamReader(in));
        int generatedId = 0;
        while (true) {
            try {
                line = logReader.readLine();
                if (line == null) {
                    logData = parser.parse("", parsingContext);
                    if (logData != null) {
                        logDataCollector.add(logData);
                    }
                    break;
                }

                logData = parser.parse(line, parsingContext);
                if (logData != null) {
                    logData.setId(generatedId++);
                    logDataCollector.add(logData);
                }

            } catch (IOException e) {
                e.printStackTrace();
                break;
            } catch (ParseException e) {
                e.printStackTrace();
                break;
            }
        }
        if (addAllDataOnFinish) {
            LogData[] data = logDataCollector.getLogData();
            dataTableModel.add(data);
        }

    }

    @Override
    public String getDisplayableName() {
        return displayableName;
    }

    @Override
    public String getKeyStrokeAccelelator() {
        return keyStrokeAccelelator;
    }

    @Override
    public int getMnemonic() {
        return mnemonic;
    }

    @Override
    public Icon getIcon() {
        return icon;
    }

}
