package pl.otros.logview.importer;

import java.io.IOException;
import java.io.InputStream;

public class ObserableInputStreamImpl extends InputStream implements ObservableStream {

    private InputStream src;
    private int current = 0;

    public ObserableInputStreamImpl(InputStream src) {
        super();
        this.src = src;
    }

    @Override
    public int read() throws IOException {
        current++;
        return src.read();
    }

    public int available() throws IOException {
        return src.available();
    }

    public void close() throws IOException {
        src.close();
    }

    public void mark(int readlimit) {
        src.mark(readlimit);
    }

    public boolean markSupported() {
        return src.markSupported();
    }

    public int read(byte[] b, int off, int len) throws IOException {
        int read = src.read(b, off, len);
        current += read;
        return read;
    }

    public int read(byte[] b) throws IOException {
        int read = src.read(b);
        current += read;
        return read;
    }

    public void reset() throws IOException {
        src.reset();
    }

    public long skip(long n) throws IOException {
        return src.skip(n);
    }

    /*
     * (non-Javadoc)
     * 
     * @see pl.otros.logview.importer.ni.ObservableStream#getCurrentRead()
     */
    public long getCurrentRead() {
        return current;
    }

    /*
     * (non-Javadoc)
     * 
     * @see pl.otros.logview.importer.ni.ObservableStream#getMax()
     */
    public long getMax() throws IOException {
        return current + src.available();
    }

}
