package pl.otros.logview.loader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import pl.otros.logview.gui.markers.AutomaticMarker;
import pl.otros.logview.gui.markers.RegexMarker;
import pl.otros.logview.gui.markers.StringMarker;

public class AutomaticMarkerLoader {

    public static ArrayList<AutomaticMarker> loadInternalMarkers() throws IOException {

        ArrayList<AutomaticMarker> markers = new ArrayList<AutomaticMarker>();
        BufferedReader in = new BufferedReader(new InputStreamReader(AutomaticMarkerLoader.class.getClassLoader().getResourceAsStream(
                "markers.txt")));
        if (in == null) {
            throw new IOException("Null");
        }
        String line = null;
        while ((line = in.readLine()) != null) {
            if (line.startsWith("#")) {
                continue;
            }
            try {
                Class<?> c = AutomaticMarkerLoader.class.getClassLoader().loadClass(line);
                AutomaticMarker am = (AutomaticMarker) c.newInstance();
                markers.add(am);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }

        }

        return markers;

    }

    public static ArrayList<AutomaticMarker> load(File dir) {
        ArrayList<AutomaticMarker> markers = new ArrayList<AutomaticMarker>();
        if (!dir.exists()) {
            // dir not exist!
            return markers;
        }
        File[] files = dir.listFiles(new FileFilter() {

            @Override
            public boolean accept(File pathname) {
                if (pathname.isDirectory() || pathname.getName().endsWith(".jar") || pathname.getName().endsWith(".zip")) {
                    return true;
                }
                return false;
            }

        });
        for (int i = 0; i < files.length; i++) {
            ArrayList<AutomaticMarker> m = null;
            try {
                if (files[i].isDirectory()) {
                    m = loadFromDir(files[i]);
                } else {
                    m = loadFromJar(files[i]);
                }
                markers.addAll(m);

            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        return markers;
    }

    private static ArrayList<AutomaticMarker> loadFromJar(File file) throws IOException, ClassNotFoundException {
        ArrayList<AutomaticMarker> markers = new ArrayList<AutomaticMarker>();
        List<Class<?>> implementationClasses = LvDynamicLoader.getInstance().getInterfaceImplementations(AutomaticMarker.class, file);

        for (Class<?> class1 : implementationClasses) {
            try {
                AutomaticMarker am = (AutomaticMarker) class1.newInstance();
                markers.add(am);
            } catch (InstantiationException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        }

        return markers;
    }

    private static ArrayList<AutomaticMarker> loadFromDir(File file) {

        return null;
    }

    public static ArrayList<AutomaticMarker> loadRegexMarkers(File dir) {
        ArrayList<AutomaticMarker> markers = new ArrayList<AutomaticMarker>();
        File[] files = dir.listFiles(new FileFilter() {
            @Override
            public boolean accept(File pathname) {
                return pathname.isFile() && pathname.getName().endsWith(".regexMarker");
            }
        });
        if (files != null) {
            for (File file : files) {
                try {
                    markers.add(loadRegexMarkerFromProperties(file));
                } catch (Exception e) {
                    System.err.println("Cannot initalize RegexMarker from file " + file.getName());
                }
            }
        }
        return markers;
    }
    
    public static ArrayList<AutomaticMarker> loadStringMarkers(File dir) {
        ArrayList<AutomaticMarker> markers = new ArrayList<AutomaticMarker>();
        File[] files = dir.listFiles(new FileFilter() {
            @Override
            public boolean accept(File pathname) {
                return pathname.isFile() && pathname.getName().endsWith(".stringMarker");
            }
        });
        if (files != null) {
            for (File file : files) {
                try {
                    markers.add(loadStringMarkerFromProperties(file));
                } catch (Exception e) {
                    System.err.println("Cannot initalize StringMarker from file " + file.getName());
                }
            }
        }
        return markers;
    }

    private static AutomaticMarker loadRegexMarkerFromProperties(File file) throws Exception {
        Properties p = new Properties();
        p.load(new FileInputStream(file));
        RegexMarker marker = new RegexMarker(p);
        return marker;
    }
    
    private static AutomaticMarker loadStringMarkerFromProperties(File file) throws Exception {
        Properties p = new Properties();
        p.load(new FileInputStream(file));
        StringMarker marker = new StringMarker(p);
        return marker;
    }
    
}
