package pl.otros.logview.loader;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

import pl.otros.logview.importer.InitializationException;
import pl.otros.logview.importer.LogImporter;
import pl.otros.logview.importer.LogImporterUsingParser;
import pl.otros.logview.importer.XMLLogImporter;
import pl.otros.logview.parser.JulSimpleFormmaterParser;
import pl.otros.logview.parser.LogParser;

public class LogImportersLoader {

    public static Collection<LogImporter> loadInternalLogImporters() throws InitializationException {
        ArrayList<LogImporter> list = new ArrayList<LogImporter>();

        Properties p = new Properties();

        XMLLogImporter xmlLogImporter = new XMLLogImporter();
        xmlLogImporter.init(p);
        list.add(xmlLogImporter);
        
        JulSimpleFormmaterParser julSimpleFormmaterParser = new JulSimpleFormmaterParser();
        LogImporterUsingParser julImporter = new LogImporterUsingParser(julSimpleFormmaterParser);
        julImporter.init(p);
        list.add(julImporter);
        
        return list;

    }

    public static ArrayList<LogImporter> load(File dir) {
        ArrayList<LogImporter> logImporters = new ArrayList<LogImporter>();
        if (!dir.exists()) {
            // dir not exist!
            return logImporters;
        }
        File[] files = dir.listFiles(new FileFilter() {

            @Override
            public boolean accept(File pathname) {
                if (pathname.isDirectory() || pathname.getName().endsWith(".jar") || pathname.getName().endsWith(".zip")) {
                    return true;
                }
                return false;
            }

        });
        for (int i = 0; i < files.length; i++) {
            ArrayList<LogImporter> m = null;
            try {
                if (files[i].isDirectory()) {
                    m = loadFromDir(files[i]);
                } else {
                    m = loadFromJar(files[i]);
                }
                logImporters.addAll(m);

            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        return logImporters;
    }

    private static ArrayList<LogImporter> loadFromJar(File file) throws IOException, ClassNotFoundException {
        ArrayList<LogImporter> importers = new ArrayList<LogImporter>();
        List<Class<?>> implementationClasses = LvDynamicLoader.getInstance().getInterfaceImplementations(LogImporter.class, file);

        for (Class<?> class1 : implementationClasses) {
            try {
                LogImporter am = (LogImporter) class1.newInstance();
                importers.add(am);
            } catch (InstantiationException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        }

        List<Class<?>> implementationClasses2 = LvDynamicLoader.getInstance().getInterfaceImplementations(LogParser.class, file);

        for (Class<?> class1 : implementationClasses2) {
            try {
                System.out.println("LogImportersLoader.loadFromJar() parser class: " + class1.getName());
                LogParser am = (LogParser) class1.newInstance();
                System.out.println("LogImportersLoader.loadFromJar() parser: " + am);
                LogImporterUsingParser l = new LogImporterUsingParser(am);
                importers.add(l);
            } catch (InstantiationException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        }

        return importers;
    }

    private static ArrayList<LogImporter> loadFromDir(File file) {

        return null;
    }
}
