package pl.otros.logview.loader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import pl.otros.logview.filter.LogFilter;
import pl.otros.logview.gui.markers.AutomaticMarker;
import pl.otros.logview.importer.InitializationException;
import pl.otros.logview.importer.LogImporter;


public class LvDynamicLoader {

    private static LvDynamicLoader instance = null;

    public static LvDynamicLoader getInstance() {
        if (instance == null) {
            synchronized (LvDynamicLoader.class) {
                if (instance == null) {
                    instance = new LvDynamicLoader();

                }
            }
        }
        return instance;
    }

    private Collection<AutomaticMarker> automaticMarkers;
    private Collection<LogImporter> logImporters;
    private Collection<LogFilter> logFilters;

    private LvDynamicLoader() {
        super();

    }

    public void loadAll() throws IOException, InitializationException {
        automaticMarkers = new ArrayList<AutomaticMarker>();
        logImporters = new ArrayList<LogImporter>();
        logFilters = new ArrayList<LogFilter>();
        loadAutomaticMarkers();
        loadLogFilters();
        loadLogImporters();
    }

    private void loadAutomaticMarkers() throws IOException {

        automaticMarkers.addAll(AutomaticMarkerLoader.loadInternalMarkers());
        File f = new File("plugins/markers");
        automaticMarkers.addAll(AutomaticMarkerLoader.load(f));
        System.out.println("LvDynamicLoader.loadAutomaticMarkers() "+f.getAbsolutePath());
        automaticMarkers.addAll(AutomaticMarkerLoader.loadRegexMarkers(f));
        automaticMarkers.addAll(AutomaticMarkerLoader.loadStringMarkers(f));

    }

    private void loadLogImporters() throws InitializationException {
        logImporters.addAll(LogImportersLoader.loadInternalLogImporters());
        File f = new File("plugins/logimporters");
        logImporters.addAll(LogImportersLoader.load(f));
        String userHome = System.getProperty("user.home");
        File f2 = new File(userHome + "/.lv/plugins/logimporters");
        logImporters.addAll(LogImportersLoader.load(f2));

    }

    private void loadLogFilters() {
        logFilters.addAll(LogFiltersLoader.loadInternalFilters());
    }

    public Collection<AutomaticMarker> getAutomaticMarkers() {
        return automaticMarkers;
    }

    public Collection<LogImporter> getLogImporters() {
        return logImporters;
    }

    public Collection<LogFilter> getLogFilters() {
        return logFilters;
    }
    
    public List<String> getClassesFromJar(File jarFile) throws FileNotFoundException, IOException{
        ArrayList<String> list = new ArrayList<String>();
        JarInputStream jarInputStream = new JarInputStream(new FileInputStream(jarFile));
        JarEntry jarEntry;
        while (true){
            jarEntry = jarInputStream.getNextJarEntry();
            if (null  == jarEntry){
                break;//
            }
            String s = jarEntry.getName();
            if (s.endsWith(".class")){
                list.add(s.replace('/', '.').substring(0, s.length()-6));
            }
        }
                
        return list;
    }
    
    
    public List<Class<?>> getInterfaceImplementations(Class<?> interfaceClass, File f) throws FileNotFoundException, IOException, ClassNotFoundException{
        ArrayList<Class<?>> list= new ArrayList<Class<?>>();        
        List<String> classes =getClassesFromJar(f);
        ClassLoader cl = new URLClassLoader(new URL[] { f.toURI().toURL() });
        for (String klazz : classes) {
            Class<?> c = cl.loadClass(klazz);
            Class<?>[] interf = c.getInterfaces();
            for (int i = 0; i < interf.length; i++) {
                if (interfaceClass.equals(interf[i])){
                    list.add(c);
                    break;
                }
            }
        }
        return list;
        
    }
}
