package pl.otros.logview.parser;

import java.awt.event.KeyEvent;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import pl.otros.logview.LogData;

public class JulSimpleFormmaterParser implements LogParser {
    
    private ParserDescription pd;
    private static final DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static final Pattern datePatter = Pattern.compile("(\\d{4})-(\\d{2})-(\\d{2}) (\\d{2}:)(\\d{2}:)(\\d{2})");

    public JulSimpleFormmaterParser() {
        pd = new ParserDescription();
        pd.setDisplayName("JUL simple formmater");
        pd.setDescription("Parse logs formmated by java.util.logging with SimpleFormmater");
        pd.setKeyStrokeAccelelator("control j");
        pd.setMenmonic(KeyEvent.VK_J);
    }

    @Override
    public ParserDescription getParserDescription() {
        return pd;
    }

    public LogData parse(String event, ParsingContext context) throws ParseException {
        // if (sb.length() < 24) {
        // return null;
        // }
        if (event == null || event.length() == 0) {
            return null;
        }
        StringBuilder sb = context.getUnmatchedLog();

        try {
            Matcher m = datePatter.matcher(event);
            if (!m.find()) {
                return null;
            }
            return tryToParseStringBuilder(sb);
        } catch (ParseException e) {
//            e.printStackTrace();
            return null;
        } catch (IndexOutOfBoundsException e) {
//            e.printStackTrace();
            return null;
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return null;
        } catch (Exception e) {
            System.err.println("Exception: " + e.getClass());
            System.err.println("Exception " + e.getMessage());
            System.err.println("event: " + event);
            System.err.println("Buffer: " + sb.toString());
            return null;
        } finally {
            sb.append(event);
            sb.append("\n");
        }
    }

    private LogData tryToParseStringBuilder(StringBuilder sb) throws ParseException {
        String dateString = sb.substring(0, 19);
        // 2010-02-11 16:08:06 d.bf.g.A main
        LogData logData = new LogData();
        Date date = df.parse(dateString);
        logData.setDate(date);
        int firsLineIndex = sb.indexOf("\n");
        String classMethodLevel = sb.substring(20, firsLineIndex);
        logData.setLoggerName(classMethodLevel.split(" ")[0]);        
        String clazz = classMethodLevel.split(" ")[0];
        String method = classMethodLevel.split(" ")[1];
        String levelAndMessage = sb.substring(firsLineIndex+1);
        String level = levelAndMessage.substring(0, levelAndMessage.indexOf(':'));
        logData.setMethod(method);
        logData.setClazz(clazz);
        logData.setMessage(levelAndMessage.substring(levelAndMessage.indexOf(':')+1).trim());
        try {
            logData.setLevel(parseLevel(level));
        } catch (Exception e) {
            // Ignore logdata
            return null;
        } finally {
            sb.setLength(0);
        }
        return logData;

    }

    private Level parseLevel(String levelRAS) throws UnknownLevelException {
        Level l = Level.INFO;
        if (levelRAS.equalsIgnoreCase("FINEST")) {
            l = Level.FINEST;
        } else if (levelRAS.equalsIgnoreCase("FINER")) {
            l = Level.FINER;
        } else if (levelRAS.equalsIgnoreCase("FINE")) {
            l = Level.FINE;
        } else if (levelRAS.equalsIgnoreCase("INFO")) {
            l = Level.INFO;
        } else if (levelRAS.equalsIgnoreCase("CONFIG")) {
            l = Level.CONFIG;
        } else if (levelRAS.equalsIgnoreCase("WARNING")) {
            l = Level.WARNING;
        } else if (levelRAS.equalsIgnoreCase("SEVERE")) {
            l = Level.SEVERE;
        } else {
            throw new UnknownLevelException(levelRAS);
        }
        return l;
    }
    
}
