package pl.otros.logview.reader;

import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;

import javax.swing.SwingUtilities;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import pl.otros.logview.gui.FrameManger;
import pl.otros.logview.gui.LogDataTableModel;
import pl.otros.logview.gui.LogViewInternalFrame;
import pl.otros.logview.importer.XMLLogImporter;

public class SocketLogReader {

    private ServerSocket serverSocket;

    public void close() throws IOException {
        if (serverSocket != null) {
            serverSocket.close();
            serverSocket = null;
        }
    }

    public void start(int port) throws Exception {
        serverSocket = new ServerSocket(port);
        Runnable r = new Runnable() {

            @Override
            public void run() {
                try {
                    while (true) {
                        Socket s = serverSocket.accept();
                        SocketHandler handler = new SocketHandler(s);
                        Thread t = new Thread(handler, "Socket handler: " + s.getInetAddress() + ":" + s.getPort());
                        t.setDaemon(true);
                        t.start();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        };
        Thread t = new Thread(r, "Socket listener");
        t.setDaemon(true);
        t.start();

    }

    public boolean isClosed() {
        return serverSocket == null || serverSocket.isClosed();
    }

    private class SocketHandler implements Runnable {

        public Socket socket;

        public SocketHandler(Socket socket) {
            super();
            this.socket = socket;
        }

        @Override
        public void run() {
            String adress = socket.getInetAddress().toString() + ":" + socket.getPort();
            final LogViewInternalFrame frame = FrameManger.getInstance().createFrame(adress);
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    frame.switchToContentView();
                }
            });

            try {
                InputStream in = socket.getInputStream();
                frame.getIframe().addInternalFrameListener(new InternalFrameAdapter() {
                    @Override
                    public void internalFrameClosed(InternalFrameEvent e) {
                        try {
                            socket.close();
                        } catch (IOException e1) {
                            e1.printStackTrace();
                        }
                    }

                });

                LogDataTableModel dataTableModel = frame.getDataTableModel();
                XMLLogImporter logImporter = new XMLLogImporter();
                logImporter.importLogs(in, dataTableModel, false);
                frame.getIframe().setTitle(adress + " - connection finished ");
            } catch (IOException e) {
                e.printStackTrace();
                frame.getIframe().setTitle(adress + " - connection broken: " + e.getMessage());
            }
        }

    }
}
