package pl.otros.logview.uml;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Message {

    private MessageType type;
    private String message = "";
    private String values = "";
    private static Pattern getExitValuesPattern = Pattern.compile("Exit, return value = (.*)");

    public Message(String m) {
        m = m.trim().toLowerCase();
        if (m.startsWith("entry")) {
            type = MessageType.TYPE_ENTRY;
            message = m;

        } else if (m.startsWith("exit") || m.startsWith("return")) {
            type = MessageType.TYPE_EXIT;
            // Exit, return value = false
            Matcher match = getExitValuesPattern.matcher(m);
            if (match.find() && match.groupCount() == 1) {
                message = match.group(1);
            }

        } else {
            type = MessageType.TYPE_LOG;
            message = m;
        }
        if (message.length() > 90) {
            message = message.substring(0, 90);
        }

    }

    public static class MessageType {
        public static final MessageType TYPE_ENTRY = new MessageType("ENTRY");
        public static final MessageType TYPE_EXIT = new MessageType("EXIT");
        public static final MessageType TYPE_LOG = new MessageType("LOG");

        private String type = "";

        private MessageType(String type) {
            this.type = type;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof MessageType) {
                MessageType mt = (MessageType) obj;
                return mt.type.equalsIgnoreCase(type);
            }
            return super.equals(obj);
        }

        @Override
        public int hashCode() {
            return type.hashCode();
        }

        public String toString() {
            return type;
        }

    }

    public MessageType getType() {
        return type;
    }

    public String getMessage() {
        return message;
    }

    public String toString() {
        return type + ": " + message;
    }
}
